--- //.git/fs/object/2f8a59f4b5bfe028c022855acc19666d69eed909/tree//sys/src/9/port/devmouse.c
+++ /sys/src/9/port/devmouse.c
@@ -18,6 +18,8 @@
 	ScrollRight = 0x40,
 };
 
+char Eclip[] = "invalid clip rectangle";
+
 typedef struct Mouseinfo	Mouseinfo;
 typedef struct Mousestate	Mousestate;
 
@@ -53,6 +55,8 @@
 	CMbuttonmap,
 	CMscrollswap,
 	CMswap,
+	CMclip,
+	CMunclip,
 	CMblank,
 	CMblanktime,
 	CMtwitch,
@@ -64,6 +68,8 @@
 	CMbuttonmap,	"buttonmap",	0,
 	CMscrollswap,	"scrollswap",	0,
 	CMswap,		"swap",		1,
+	CMclip,		"clip",		0,
+	CMunclip,	"unclip",	1,
 	CMblank,	"blank",	1,
 	CMblanktime,	"blanktime",	2,
 	CMtwitch,	"twitch",	1,
@@ -95,6 +101,9 @@
 static uchar buttonmap[8] = {
 	0, 1, 2, 3, 4, 5, 6, 7,
 };
+static Rectangle clipr;
+static Chan *clipchan;
+static int clip;
 static int mouseswap;
 static int scrollswap;
 static ulong mousetime;
@@ -206,6 +215,10 @@
 	if((c->qid.type&QTDIR)!=0 || (c->flag&COPEN)==0)
 		return;
 	switch((ulong)c->qid.path){
+	case Qmousectl:
+		if(c == clipchan)
+			clip = 0;
+		break;
 	case Qmousein:
 		mouse.inbuttons &= ~((Mousestate*)c->aux)->buttons;
 		free(c->aux);	/* Mousestate */
@@ -344,6 +357,7 @@
 mousewrite(Chan *c, void *va, long n, vlong)
 {
 	char *p;
+	Rectangle r;
 	Point pt;
 	Cmdbuf *cb;
 	Cmdtab *ct;
@@ -403,6 +417,37 @@
 			mouseblankscreen(1);
 			break;
 
+		case CMclip:
+			if(cb->nf < 5)
+				error(Eshort);
+			if(!gscreen)
+				break;
+			r.min.x = strtol(cb->f[1], &p, 0);
+			if(p == nil)
+				error(Eclip);
+			r.min.y = strtol(cb->f[2], &p, 0);
+			if(p == nil)
+				error(Eclip);
+			r.max.x = strtol(cb->f[3], &p, 0);
+			if(p == nil)
+				error(Eclip);
+			r.max.y = strtol(cb->f[4], &p, 0);
+			if(p == nil)
+				error(Eclip);
+			if(!rectinrect(r, gscreen->clipr))
+				error(Eclip);
+			clipr = r;
+			clipchan = c;
+			clip = 1;
+			if(!ptinrect(mouse.xy, clipr))
+				absmousetrack(mouse.xy.x, mouse.xy.y, 0, 0);
+			break;
+
+		case CMunclip:
+			clip = 0;
+			clipchan = nil;
+			break;
+
 		case CMblanktime:
 			blanktime = strtoul(cb->f[1], 0, 0);
 			/* wet floor */
@@ -609,6 +654,17 @@
 		y = gscreen->clipr.min.y;
 	if(y >= gscreen->clipr.max.y)
 		y = gscreen->clipr.max.y-1;
+
+	if(clip){
+		if(x < clipr.min.x)
+			x = clipr.min.x;
+		if(x >= clipr.max.x)
+			x = clipr.max.x-1;
+		if(y < clipr.min.y)
+			y = clipr.min.y;
+		if(y >= clipr.max.y)
+			y = clipr.max.y-1;
+	}
 
 
 	ilock(&mouse);
